/*
 * flowlimit.c:
 *
 * Copyright (c) 2002 James McKenzie <james@fishsoup.dhs.org>,
 * All rights reserved.
 *
 */

static char rcsid[] =
  "$Id: flowlimit.c,v 1.1 2002/07/24 21:51:14 root Exp root $";

/*
 * $Log: flowlimit.c,v $
 * Revision 1.1  2002/07/24 21:51:14  root
 * Initial revision
 *
 * Revision 1.1  2002/04/10 17:13:23  root
 * #
 *
 */

#include <stdio.h>
#include <fcntl.h>
#include <time.h>
#include <sys/time.h>
#include <malloc.h>
#include <math.h>

int bs = 1;
char *buf;

int
do_time (char *ptr, double t)
{
  int ret = 0, i;
  double h, m, s;
  if (t < 60.0)
    {
      return sprintf (ptr, "%.1fs", t);
    }

  if (t < 3600.0)
    {
      m = floor (t / 60.0);
      t -= 60 * m;
      s = floor (t);
      t -= s;
      t *= 10;
      return sprintf (ptr, "%d:%02d.%dm", (int) m, (int) s, (int) t);
    }

  h = floor (t / 3600.0);
  t -= 3600 * h;
  m = floor (t / 60.0);
  t -= 60 * m;
  s = floor (t);
  t -= s;
  t *= 10;
  return sprintf (ptr, "%d:%02d:%02d.%dh", (int) h, (int) m, (int) s,
                  (int) t);
}

int
do_rate (char *ptr, double r)
{
  int ret = 0, i;
  double h, m;
  if (r < 1024.0)
    {
      return sprintf (ptr, "%.1f b", r);
    }

  if (r < 1048576.0)
    {
      return sprintf (ptr, "%.1f K", r / 1024.0);
    }
  return sprintf (ptr, "%.1f M", r / 1048576.0);
}




int
main (int argc, char *argv[])
{
  struct timeval tv1, tv2, tv3, tv4, qlasttime, lasttime, starttime;
  int red;
  long unsigned bytes, lastbytes;
  double wantrate = (double) atoi (argv[1]);

  buf = malloc (bs);

  bytes = 0;
  gettimeofday (&starttime, NULL);

  lastbytes = 0;
  gettimeofday (&lasttime, NULL);
  gettimeofday (&qlasttime, NULL);

  do
    {
      gettimeofday (&tv1, NULL);
      red = read (0, buf, bs);
      if (red <= 0)
        break;
      write (1, buf, red);
      gettimeofday (&tv2, NULL);

      bytes += red;

      timersub (&tv2, &tv1, &tv3);

      if ((tv3.tv_sec > 0) && (bs > 1))
        {
          bs = bs >> 1;
          buf = realloc (buf, bs);
        }

      if ((tv3.tv_sec == 0) && (tv3.tv_usec < 100000) && (bs < (8 * 1024)))
        {
          bs = bs << 1;
          buf = realloc (buf, bs);
        }


      timersub (&tv2, &lasttime, &tv3);
      {
        char charbuf[128], *ptr = charbuf;

        double totaltime, totalbytes, intertime, interbytes;
        double wanttime;

        timersub (&tv2, &starttime, &tv1);

        intertime = (double) tv3.tv_usec;
        intertime /= 1000000.0;
        intertime += (double) tv3.tv_sec;

        totaltime = (double) tv1.tv_usec;
        totaltime /= 1000000.0;
        totaltime += (double) tv1.tv_sec;

        totalbytes = (double) bytes;
        interbytes = totalbytes - (double) lastbytes;

        lastbytes = bytes;
        lasttime = tv2;
        intertime += 0.001;
        totaltime += 0.001;

        wanttime = totalbytes / wantrate;
        wanttime -= totaltime;


        timersub (&tv2, &qlasttime, &tv4);
        if ((tv4.tv_sec > 5))
          {
            qlasttime = tv2;
            ptr += do_time (ptr, totaltime);
            ptr += sprintf (ptr, " rate:");
            ptr += do_rate (ptr, interbytes / intertime);
            ptr += sprintf (ptr, "/");
            ptr += do_rate (ptr, totalbytes / totaltime);
            ptr += sprintf (ptr, " bytes:");
            ptr += do_rate (ptr, totalbytes);
            ptr += sprintf (ptr, " (read size %d)\n", bs);

            fputs (charbuf, stderr);
          }

        if (wanttime > 0)
          {
            struct timeval wait;


            wait.tv_sec = (int) floor (wanttime);
            wanttime -= (double) wait.tv_sec;
            wanttime *= 1000000.0;
            wait.tv_usec = (int) wanttime;


            select (0, NULL, NULL, NULL, &wait);
          }


      }


    }
  while (1);

  return 0;
}
